import type { KeysOfUnion } from "type-fest"

export type DefaultProps =
  | { default?: true }
export type Default = KeysOfUnion<DefaultProps>

export type ColorProps =
  | { primary?: true}
  | { secondary?: true }
  | { destructive?: true }
export type Color = KeysOfUnion<ColorProps>

export type PastelProps =
  | { red?:true }
  | { blue?:true }
  | { purple?:true }
  | { green?: true }
  | { yellow?:true }
export type Pastel = KeysOfUnion<PastelProps>

export type VariantProps =
  | { solid?:true }
  | { outline?:true }
  | { ghost?:true }
export type Variant = KeysOfUnion<VariantProps>

export type InteractiveProps =
| { interactive?: true }
export type Interactive = KeysOfUnion<DefaultProps>

export type RaisedProps =
| { raised?: true }
export type Raised = KeysOfUnion<RaisedProps>

export type Props =
  DefaultProps & ColorProps & PastelProps & VariantProps & InteractiveProps & RaisedProps

export type ColorSelector =
  `${Color | Pastel | Default}${'' | ` ${Variant}${'' | ' interactive'}${'' | ' raised'}`}`

/**
 * Apply a color class for setting `color`, `background-color` and `border` styles.
 * You can override it with `style="..."`.
 *
 * @param color Choose a semantic color or the default page background.
 * This will affect the text color.
 * To add an outline or filled background, add a `Variant`: `'solid' | 'outline'`.
 * If the surface should stand out, add `raised`.
 * If the surface reacts to mouse input, add `interactive`.
 * @returns the corresponding `class` object
 *
 * Note: Make sure to implement the necessary classes in `colors.scss`!
 */
export const color = (color: ColorSelector) =>
  ({ class: color })

// Color from Props
export const propsToColor = (props: Partial<Props>) =>
  ({ class: Object.entries(props).filter(([key, value])=>value && key).map(([key,_])=>key).join(" ") })
